package ch.fritteli.labyrinth.generator.renderer.htmlfile;

import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.renderer.Renderer;
import ch.fritteli.labyrinth.generator.renderer.html.HTMLRenderer;
import io.vavr.control.Option;
import io.vavr.control.Try;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.NoSuchElementException;

@Slf4j
public class HTMLFileRenderer implements Renderer<Path> {
    @NonNull
    private static final HTMLRenderer HTML_RENDERER = HTMLRenderer.newInstance();
    @NonNull
    private Option<Path> targetFile;

    private HTMLFileRenderer() {
        this.targetFile = Try
                .of(() -> Files.createTempFile("labyrinth_", ".html"))
                .onFailure(ex -> log.error("Unable to set default target file.", ex))
                .toOption();
    }

    @NonNull
    public static HTMLFileRenderer newInstance() {
        return new HTMLFileRenderer();
    }

    public boolean isTargetFileDefinedAndWritable() {
        return this.targetFile
                .map(Path::toFile)
                .exists(File::canWrite);
    }

    @NonNull
    public HTMLFileRenderer setTargetFile(@NonNull final Path targetFile) {
        this.targetFile = Option.of(targetFile);
        return this;
    }

    @NonNull
    @Override
    public Path render(@NonNull final Labyrinth labyrinth) {
        if (!this.isTargetFileDefinedAndWritable()) {
            try {
                Files.createFile(this.targetFile.get());
            } catch (IOException | NoSuchElementException e) {
                throw new IllegalArgumentException("Cannot write to target file.", e);
            }
        }
        final String html = HTML_RENDERER.render(labyrinth);
        final Path targetFile = this.targetFile.get();
        try {
            Files.writeString(targetFile, html, StandardCharsets.UTF_8);
        } catch (IOException e) {
            log.error("Failed writing to file " + targetFile.normalize(), e);
        }
        return targetFile;
    }
}
