package ch.fritteli.labyrinth.generator.renderer.json;

import ch.fritteli.labyrinth.generator.json.JsonCell;
import ch.fritteli.labyrinth.generator.json.JsonLabyrinth;
import ch.fritteli.labyrinth.generator.model.Direction;
import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.model.Tile;
import java.util.ArrayList;
import java.util.List;
import lombok.AccessLevel;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor(access = AccessLevel.PACKAGE)
class Generator {

    @NonNull
    private final Labyrinth labyrinth;

    @NonNull
    JsonLabyrinth generate() {
        final JsonLabyrinth result = new JsonLabyrinth();
        result.setId(this.labyrinth.getRandomSeed());
        result.setWidth(this.labyrinth.getWidth());
        result.setHeight(this.labyrinth.getHeight());
        final List<List<JsonCell>> rows = new ArrayList<>();
        for (int y = 0; y < this.labyrinth.getHeight(); y++) {
            final ArrayList<JsonCell> row = new ArrayList<>();
            for (int x = 0; x < this.labyrinth.getWidth(); x++) {
                // x and y are not effectively final and can therefore not be accessed from within the lambda. Hence, create the string beforehand.
                final String exceptionString = "Failed to obtain tile at %dx%d, although labyrinth has dimensoins %dx%d"
                        .formatted(x, y, this.labyrinth.getWidth(), this.labyrinth.getHeight());
                final Tile tile = this.labyrinth.getTileAt(x, y)
                        .getOrElseThrow(() -> new IllegalStateException(exceptionString));
                final JsonCell cell = new JsonCell();
                cell.setTop(tile.hasWallAt(Direction.TOP));
                cell.setRight(tile.hasWallAt(Direction.RIGHT));
                cell.setBottom(tile.hasWallAt(Direction.BOTTOM));
                cell.setLeft(tile.hasWallAt(Direction.LEFT));
                cell.setSolution(tile.isSolution());
                row.add(cell);
            }
            rows.add(row);
        }
        result.setGrid(rows);
        return result;
    }
}
