package ch.fritteli.labyrinth.generator.renderer.json;

import ch.fritteli.labyrinth.generator.json.JsonCell;
import ch.fritteli.labyrinth.generator.json.JsonLabyrinth;
import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.renderer.Renderer;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import java.util.ArrayList;
import java.util.List;
import lombok.NonNull;

public class JsonRenderer implements Renderer<String> {

    @NonNull
    private final ObjectMapper objectMapper;

    private JsonRenderer() {
        this.objectMapper = new ObjectMapper()
                .enable(SerializationFeature.INDENT_OUTPUT);
    }

    @NonNull
    public static JsonRenderer newInstance() {
        return new JsonRenderer();
    }

    @NonNull
    private static JsonLabyrinth createSingleCellLabyrinth() {
        // This is the only cell.
        final JsonCell cell = new JsonCell();
        cell.setRight(true);
        cell.setLeft(true);
        cell.setSolution(true);
        // Wrap that in a nested list.
        final List<List<JsonCell>> rows = new ArrayList<>();
        rows.add(new ArrayList<>());
        rows.get(0).add(cell);
        // Wrap it all in an instance of JsonLabyrinth.
        final JsonLabyrinth jsonLabyrinth = new JsonLabyrinth();
        jsonLabyrinth.setId(0L);
        jsonLabyrinth.setGrid(rows);
        return jsonLabyrinth;
    }

    @NonNull
    private String toString(@NonNull final JsonLabyrinth jsonLabyrinth) {
        try {
            return this.objectMapper.writeValueAsString(jsonLabyrinth);
        } catch (final JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    @NonNull
    @Override
    public String render(@NonNull final Labyrinth labyrinth) {
        final JsonLabyrinth jsonLabyrinth;
        if (labyrinth.getWidth() == 0 || labyrinth.getHeight() == 0) {
            jsonLabyrinth = createSingleCellLabyrinth();
        } else {
            final Generator generator = new Generator(labyrinth);
            jsonLabyrinth = generator.generate();
        }
        return toString(jsonLabyrinth);
    }
}
