package ch.fritteli.labyrinth.generator.serialization;

import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.model.Tile;
import lombok.NonNull;

import java.io.ByteArrayInputStream;

public class LabyrinthInputStream extends ByteArrayInputStream {
    public LabyrinthInputStream(@NonNull final byte[] buf) {
        super(buf);
    }

    public long readLong() {
        long result = 0;
        result |= ((long) this.readInt()) << 32;
        result |= 0xffffffffL & this.readInt();
        return result;
    }

    public int readInt() {
        int result = 0;
        result |= (0xff & this.readByte()) << 24;
        result |= (0xff & this.readByte()) << 16;
        result |= (0xff & this.readByte()) << 8;
        result |= 0xff & this.readByte();
        return result;
    }

    public void checkHeader() {
        final byte magic1 = this.readByte();
        if (magic1 != SerializerDeserializer.MAGIC_BYTE_1) {
            throw new IllegalArgumentException("Invalid labyrinth data.");
        }
        final byte magic2 = this.readByte();
        if (magic2 != SerializerDeserializer.MAGIC_BYTE_2) {
            throw new IllegalArgumentException("Invalid labyrinth data.");
        }
        final int version = this.readByte();
        if (version != SerializerDeserializer.VERSION_BYTE) {
            throw new IllegalArgumentException("Unknown Labyrinth data version: " + version);
        }
    }

    public byte readByte() {
        final int read = this.read();
        if (read == -1) {
            // end of stream reached
            throw new ArrayIndexOutOfBoundsException("End of stream reached. Cannot read more bytes.");
        }
        return (byte) read;
    }

    @NonNull
    public Labyrinth readLabyrinthData() {
        final long randomSeed = this.readLong();
        final int width = this.readInt();
        final int height = this.readInt();

        final Tile[][] tiles = new Tile[width][height];
        for (int x = 0; x < width; x++) {
            tiles[x] = new Tile[height];
        }

        for (int y = 0; y < height; y++) {
            for (int x = 0; x < width; x++) {
                final byte bitmask = this.readByte();
                tiles[x][y] = SerializerDeserializer.getTileForBitmask(bitmask);
            }
        }

        return SerializerDeserializer.createLabyrinth(tiles, width, height, randomSeed);
    }
}
