package ch.fritteli.labyrinth.generator.serialization;

import ch.fritteli.labyrinth.generator.model.Labyrinth;
import ch.fritteli.labyrinth.generator.model.Tile;
import lombok.NonNull;

import java.io.ByteArrayOutputStream;


public class LabyrinthOutputStream extends ByteArrayOutputStream {
    public void writeHeader() {
        this.writeByte(SerializerDeserializer.MAGIC_BYTE_1);
        this.writeByte(SerializerDeserializer.MAGIC_BYTE_2);
        this.writeByte(SerializerDeserializer.VERSION_BYTE);
    }

    public void writeByte(final byte value) {
        this.write(value);
    }

    public void writeLabyrinthData(@NonNull final Labyrinth labyrinth) {
        final long randomSeed = labyrinth.getRandomSeed();
        final int width = labyrinth.getWidth();
        final int height = labyrinth.getHeight();
        this.writeLong(randomSeed);
        this.writeInt(width);
        this.writeInt(height);

        for (int y = 0; y < height; y++) {
            for (int x = 0; x < width; x++) {
                // We .get() it, because we want to crash hard if it is not available.
                final Tile tile = labyrinth.getTileAt(x, y).get();
                final byte bitmask = SerializerDeserializer.getBitmaskForTile(tile);
                this.writeByte(bitmask);
            }
        }
    }

    public void writeLong(final long value) {
        this.writeInt((int) (value >> 32));
        this.writeInt((int) value);
    }

    public void writeInt(final int value) {
        this.write(value >> 24);
        this.write(value >> 16);
        this.write(value >> 8);
        this.write(value);
    }
}
